/*
* Copyright (C) 2014 Andrew Reid and the ModelGUI Project <http://mgui.wikidot.com>
* 
* This file is part of ModelGUI[core] (mgui-core).
* 
* ModelGUI[core] is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* ModelGUI[core] is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with ModelGUI[core]. If not, see <http://www.gnu.org/licenses/>.
*/

package mgui.interfaces.shapes;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;

import mgui.geometry.Polygon2D;
import mgui.geometry.Shape2D;
import mgui.interfaces.attributes.Attribute;
import mgui.interfaces.attributes.AttributeList;
import mgui.interfaces.graphics.util.DrawingEngine;
import mgui.interfaces.maps.Map2D;
import mgui.numbers.MguiBoolean;
import mgui.numbers.MguiInteger;


public class IntPolygon2DInt extends Polygon2DInt {

	public ArrayList<MguiInteger> integerNodes;
	
	public IntPolygon2DInt() {
		super();
		init();
	}

	public IntPolygon2DInt(Polygon2D thisPoly) {
		super(thisPoly);
		init();
	}
	
	public IntPolygon2DInt(Polygon2DInt thisPoly){
		super((Polygon2D)thisPoly.thisShape);
		attributes = (AttributeList)thisPoly.attributes.clone();
		init();
	}
	
	private void init(){
		Circle2DInt trueShape = new Circle2DInt();
		trueShape.setAttribute("HasFill", new MguiBoolean(true));
		trueShape.setAttribute("FillColour", Color.RED);
		Circle2DInt falseShape = new Circle2DInt();
		trueShape.setAttribute("HasFill", new MguiBoolean(true));
		trueShape.setAttribute("FillColour", Color.GREEN);
		
		attributes.add(new Attribute("HighlightTrue", new MguiBoolean(true)));
		attributes.add(new Attribute("HighlightFalse", new MguiBoolean(false)));
		attributes.add(new Attribute("TrueShape", trueShape));
		attributes.add(new Attribute("FalseShape", falseShape));
		attributes.add(new Attribute("HighlightSize", new MguiInteger(7)));
		attributes.add(new Attribute("LabelTrue", new MguiBoolean(true)));
		attributes.add(new Attribute("LabelSize", new MguiInteger(10)));
		
		/**@TODO add shape & colour tables to determine symbol display based
		 * upon node values */
		
		resetNodes();
	}
	
	public void resetNodes(){
		integerNodes = new ArrayList<MguiInteger>();
		if (this.thisShape != null){
			for (int i = 0; i < ((Polygon2D)thisShape).nodes.size(); i++)
				integerNodes.add(new MguiInteger(0));
		}
	}
	
	@Override
	public void setShape(Shape2D thisShape){
		super.setShape(thisShape);
		init();
	}

	public void setNodeVal(int node, int val){
		if (node > -1 && node < integerNodes.size())
			integerNodes.get(node).setValue(val);
	}
	
	public int getNodeBool(int node){
		if (node > -1 && node < integerNodes.size())
			return integerNodes.get(node).getInt();
		return 0;
	}
	
	@Override
	public void drawShape2D(Graphics2D g, DrawingEngine d){
		super.drawShape2D(g, d);
		Circle2DInt thisCircle;
		int size = ((MguiInteger)attributes.getValue("HighlightSize")).getInt();
		if (((MguiBoolean)attributes.getValue("HighlightTrue")).getTrue())
			//draw true shape for all true nodes
			for (int i = 0; i < this.getPolygon().nodes.size(); i++)
				if ((integerNodes.get(i)).getInt() != 0){
					thisCircle = (Circle2DInt)attributes.getValue("TrueShape");
					thisCircle.getCircle().centerPt.x = this.getPolygon().nodes.get(i).x;
					thisCircle.getCircle().centerPt.y = this.getPolygon().nodes.get(i).y;
					thisCircle.getCircle().radius = d.getMap().getMapDist(size);
					thisCircle.drawShape2D(g, d);
					//draw label
					if (((MguiBoolean)attributes.getValue("LabelTrue")).getTrue()){
						Map2D thisMap = d.getMap();
						Point thisPt = thisMap.getScreenPoint(getPolygon().nodes.get(i));
						g.setFont(new Font("Arial",
										   Font.PLAIN,
								           ((MguiInteger)attributes.getValue("LabelSize")).getInt()));
						g.drawString((integerNodes.get(i)).toString("#0"),
									 thisPt.x + 3, thisPt.y - 3);
						}
					}
		if (((MguiBoolean)attributes.getValue("HighlightFalse")).getTrue())
			//draw true shape for all true nodes
			for (int i = 0; i < this.getPolygon().nodes.size(); i++)
				if ((integerNodes.get(i)).getInt() == 0){
					thisCircle = (Circle2DInt)attributes.getValue("FalseShape");
					thisCircle.getCircle().centerPt.x = this.getPolygon().nodes.get(i).x;
					thisCircle.getCircle().centerPt.y = this.getPolygon().nodes.get(i).y;
					thisCircle.getCircle().radius = d.getMap().getMapDist(size);
					thisCircle.drawShape2D(g, d);
					}
	}
	
	@Override
	public String toString(){
		return "IntPolygon2DInt [" + String.valueOf(ID) + "]"; 
	}
	
	
	
	
	
}